@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2018 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -51,15 +51,13 @@ package RT::Shredder::Constants;
 use strict;
 use warnings;
 
-use base qw(Exporter);
-
 =head1 NAME
 
 RT::Shredder::Constants -  RT::Shredder constants that is used to mark state of RT objects.
 
 =head1 DESCRIPTION
 
-This module exports two group of bit constants.
+This module contains two group of bit constants.
 First group is group of flags which are used to clarify dependecies between objects, and
 second group is states of RT objects in Shredder cache.
 
@@ -84,19 +82,12 @@ This flag is used to mark dependencies that can be resolved with changing
 value in target object. For example ticket can be created by user we can
 change this reference when we delete user.
 
-=head2 RELATES
-
-This flag is used to validate relationships integrity. Base object
-is valid only when all target objects which are marked with this flags
-exist.
-
 =cut
 
 use constant {
-    DEPENDS_ON    => 0x000001,
-    WIPE_AFTER    => 0x000010,
-    RELATES        => 0x000100,
-    VARIABLE    => 0x001000,
+    DEPENDS_ON => 0x001,
+    WIPE_AFTER => 0x002,
+    VARIABLE   => 0x004,
 };
 
 =head1 STATES
@@ -112,33 +103,12 @@ Objects with this state are not exist any more in DB, but perl
 object is still in memory. This state is used to be shure that
 delete query is called once.
 
-=head2 VALID
-
-Object is marked with this state only when its relationships
-are valid.
-
-=head2 INVALID
-
 =cut
 
 use constant {
-    ON_STACK    => 0x00000,
-    IN_WIPING    => 0x00001,
-    WIPED        => 0x00010,
-    VALID        => 0x00100,
-    INVALID        => 0x01000,
+    ON_STACK  => 0x000,
+    IN_WIPING => 0x010,
+    WIPED     => 0x020,
 };
 
-our @EXPORT = qw(
-        DEPENDS_ON
-        WIPE_AFTER
-        RELATES
-        VARIABLE
-        ON_STACK
-        IN_WIPING
-        WIPED
-        VALID
-        INVALID
-        );
-
 1;
